# Generated by Django 2.2.7 on 2020-02-28 03:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inputgooglesheets', '0003_spreadsheet_automatically_imported'),
    ]

    operations = [
        migrations.AddField(
            model_name='settings',
            name='autoimport_sleep',
            field=models.PositiveIntegerField(blank=True, default=30, help_text='Amount of seconds to sleep after each autoimport.', verbose_name='Autoimport sleep'),
        ),
    ]
