# Generated by Django 2.2.7 on 2020-07-22 05:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('inputgooglesheets', '0004_settings_autoimport_sleep'),
    ]

    operations = [
        migrations.AlterField(
            model_name='settings',
            name='autoimport_sleep',
            field=models.PositiveIntegerField(blank=True, default=30, help_text='Amount of seconds to sleep after each autoimport (use 0 to disable autoimport).', verbose_name='Autoimport sleep'),
        ),
        migrations.CreateModel(
            name='QuotaExceeded',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('wait_until', models.DateTimeField(editable=False, verbose_name='Waith until')),
                ('access_token', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='quotas_exceeded', to='inputgooglesheets.AccessToken', verbose_name='Access token')),
            ],
            options={
                'verbose_name': 'Quota exceeded',
                'verbose_name_plural': 'Quotas exceeded',
            },
        ),
    ]
